﻿using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace Lfoedd
{
    internal class ColorTool : MVRScript
    {
        protected Atom _selectedAtom;
        protected UIDynamicColorPicker _colorPicker;
        protected JSONStorableColor _selectedColor;
        protected UIDynamicButton _refreshButton;
        protected UIDynamicButton _selectAllButton;

        protected Dictionary<DAZClothingItem, JSONStorableBool> _toggles = new Dictionary<DAZClothingItem, JSONStorableBool>();
        protected Dictionary<DAZClothingItem, UIDynamicToggle> _clothes = new Dictionary<DAZClothingItem, UIDynamicToggle>();
        protected List<DAZClothingItem> _selectedClothes = new List<DAZClothingItem>();

        public override void Init()
        {
            _selectedAtom = containingAtom;

            if (_selectedAtom.type != "Person")
            {
                SuperController.LogError("This script can only be attached to a Person atom.");
                return;
            }

            var color = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
            _selectedColor = new JSONStorableColor("DiffuseColor", color, UpdateColors);
            RegisterColor(_selectedColor);
            _colorPicker = CreateColorPicker(_selectedColor, true);

            _refreshButton = CreateButton("Refresh", false);
            _refreshButton.height = 100;
            _refreshButton.button.onClick.AddListener(Refresh);

            Refresh();
        }

        private void Refresh()
        {
            foreach (var toggle in _clothes)
            {
                RemoveToggle(toggle.Value);
            }

            foreach (var clothingItem in GameObject.FindObjectsOfType<DAZClothingItem>().Where(x => x.containingAtom == _selectedAtom).ToList())
            {
                if (!_toggles.ContainsKey(clothingItem))
                {
                    _toggles.Add(clothingItem, new JSONStorableBool(clothingItem.name, false));
                    RegisterBool(_toggles[clothingItem]);
                }

                var toggle = CreateToggle(_toggles[clothingItem]);
                toggle.label = clothingItem.displayName;
                toggle.toggle.isOn = _selectedClothes.Contains(clothingItem);
                toggle.toggle.onValueChanged.AddListener(OnItemSelect);
                if (_clothes.ContainsKey(clothingItem)) _clothes[clothingItem] = toggle;
                else _clothes.Add(clothingItem, toggle);
            }
        }

        // Params required for the callback, not actually used.
        private void UpdateColors(float h = 0f, float s = 0f, float v = 0f)
        {
            foreach (var clothingItem in _selectedClothes)
            {
                foreach (var material in clothingItem.GetComponentsInChildren<DAZSkinWrap>().SelectMany(dsw => dsw.GPUmaterials))
                {
                    material.SetColor("_Color", Color.HSVToRGB(_selectedColor.val.H, _selectedColor.val.S, _selectedColor.val.V));
                }
            }
        }

        private void OnItemSelect(bool toggled)
        {
            _selectedClothes.Clear();
            foreach (var toggle in _clothes)
            {
                if (toggle.Value.toggle.isOn) _selectedClothes.Add(toggle.Key);
            }
            UpdateColors();
        }
    }
}